#!/usr/bin/env python3
"""
Handle API requests to the database
"""
import json
import sqlite3


# DATABASE = '/usr/local/www/mocha-server/mocha.db'
DATABASE = 'mocha.db'

conn = sqlite3.connect(DATABASE)
conn.row_factory = sqlite3.Row
cursor = conn.cursor()



# TODO: Add fetching of list of users
# TODO: Add fetching of top N users by score
# TODO: Add ability to store and retrieve avatars (as image files?)

def get_top_N(search_parameter, N):
    """
    in progress
    """
    cursor.execute("select * from users order by p0 desc limit p1", {'p0':search_parameter, 'p1':N})
    conn.commit()
    conn.close()


# add new parameters as needed
def update_row(user_id, updated_username):
    print()

def insert_row(user_id, username):
    """
	Inserts a row for a NEW user with given parameters
	This may work better with AUTOINCREMENT to avoid arbitrary ids and duplicates
	"""

    cursor.execute("insert into users values (?,?)", (user_id, username))
    conn.commit()
    conn.close()


def fetch_user(user_id):
    """
    Returns a JSON object containing the requested user
    Also can return a list of all users if user_id == "*"
    """

    if user_id != '*': # must use (?), (item,) format
        cursor.execute("SELECT * FROM users WHERE user_id=(?)", (user_id,))
    else:
        cursor.execute("SELECT * FROM users")

    output = cursor.fetchall()
    output = json.dumps([dict(row) for row in output])
    conn.close()

    if output == '[]':
        output = None

    return output


def process_request(uri):
    """
    Handles the API endpoint.
    Currently supports:
        -   /mocha/users/"user_id"      Returns JSON of the specified user
        -   /mocha/users/*              Returns JSON list of all users
    """
    parts = uri.split('/')[1:]
    assert parts[0] == 'mocha'

    output = None

    if parts[1] == 'users':
        output = fetch_user(parts[2])

    return output



def application(environ, start_response):
    """
    mod_wsgi entry point
    """
    status = '200 OK'
    output = process_request(environ['REQUEST_URI'])

    if output is None:
        status = '404 Not Found'
        output = ''

    output = output.encode('UTF-8')

    response_headers = [('Content-type', 'application/json'),
                        ('Content-Length', str(len(output)))]

    start_response(status, response_headers)

    return [output]
